/* global Bdf,HTMLInputElement,HTMLSelectElement,HTMLTextAreaElement,$$ */
/**
 * Objet global définissant l'espace de nom Ficheform
 * 
 * @namespace Ficheform
 */
var Ficheform = {};

Ficheform.ARGS = {
  noTab: false,
  syntaxActive: false,
  permissions: {
  }
};
Ficheform.JsManagerInclude = ["_ @title_corpus@", "_ @title_thesaurus@", "_ @title_addenda@", "_ @corpus_newfiche@"];
Ficheform.hasChanges = false;
Ficheform.onSubmit = false;
Ficheform.timerMap = new Map();

Ficheform.init = function () {
    Bdf.$({ficheformRole: "submit-button"}).prop("disabled", true);
    var $form = Bdf.$({ficheformRole: "form"}).on('submit', function () {
        Ficheform.onSubmit = true;
    });
    Bdf.addChangeTestFunction(function () {
        return Ficheform.hasChanges;
    });
    Bdf.initBeforeUnload({
        testFunction: function () {
            return (Ficheform.hasChanges) && (!Ficheform.onSubmit);
        }
    });
    Bdf.startTimer(function () {
        if (Ficheform.hasChanges) {
            return;
        }
        if (Bdf.checkFormChange(Bdf.$({ficheformRole: "form"}))) {
            Ficheform.signalChange();
        }
    });
    for(let element of $form[0].elements) {
        let focusable = false;
        if (element instanceof HTMLInputElement) {
            switch(element.type) {
                case "radio":
                case "checkbox":
                case "text":
                    focusable = true;
                    
            }
        } else if (element instanceof HTMLSelectElement) {
            focusable = true;
        } else if (element instanceof HTMLTextAreaElement) {
            focusable = true;
        }
        if (focusable) {
            $(element).focus();
            break;
        }
    }
};

Ficheform.getOption = function (name) {
    if (name === "noTab") {
        return Ficheform.ARGS.noTab;
    }
    return false;
};

/**
 * Retourne sous la forme d'un objet JQuery l'entrée correspondant à la clé
 * en argument.
 * 
 * @param {String} key Clé
 * @returns {JQuery} Objet JQuery correspondant
 */
Ficheform.$entry = function (key) {
    return Bdf.$({ficheformRole: "entry", ficheformKey: key});
};

Ficheform.signalChange = function () {
    if (!Ficheform.hasChanges) {
        Ficheform.hasChanges = true;
        Bdf.$({ficheformRole: "submit-button"}).prop("disabled", false);
        Bdf.clearTimer();
    }
};

Ficheform.getPoidsString = function (value) {
    var idx = value.indexOf("<");
    if (idx === -1) return "";
    return value.substring(idx);
};

Ficheform.getPoids = function (value) {
    var idx = value.indexOf("<");
    if (idx === -1) return "";
    var poids = value.substring(idx+1, value.length -1);
    return poids;
};

Ficheform.checkPoids = function (value) {
    value = $.trim(value);
    if (value.length === 0) return 1;
    if (_is_int()) {
        if (value > 0) return value;
        else return false;
    }
    else return false;
    
    
    function _is_int(){
        if((parseFloat(value) == parseInt(value)) && !isNaN(parseInt(value))) {
            return true;
        } else {
            return false;
        }
    }
};

Ficheform.updateCheckBox = function (checkBox, poids) {
    var currentValue = checkBox.value;
    var idx = currentValue.indexOf("<");
    if (idx > -1) {
        currentValue = currentValue.substring(0, idx);
    }
    if (poids > 1) {
        checkBox.value = currentValue + "<" + poids + ">";
    } else  {
        checkBox.value = currentValue;
    }
};

Ficheform.openDialog = function (dialogUrl, dialogName, width, height) {
    var w = window.open(dialogUrl,dialogName,"resizable=yes,width=" + width + ",height=" + height + ",scrollbars");
    w.focus();
};

Ficheform.removeFromValue = function(value, id, separator) {
    var idString = String(id);
    var array = value.split(";");
    var newVal = "";
    for(let token of array) {
        if (separator) {
            let idx = token.indexOf(separator);
            if (idx > 0) {
                let tokenId = token.substring(0, idx);
                if (tokenId !== idString) {
                    newVal += token + ";";
                }
            }
        } else {
            if (token !== idString) {
                newVal += token + ";";
            }
        }
    }
    return newVal;
};

Ficheform.initPoidsInput = function ($itemList) {
    let buttonId = Bdf.generateId();
    $itemList.append(Bdf.render("ficheform:pioche", {
        buttonId: buttonId,
        action: "action-ficheform-Poids_on",
        locKey: "_ link.edition.poids_on",
        noTab: Ficheform.getOption("noTab")
    }));
    $$(buttonId).data("state", "on").click(function () {
       let $this = $(this);
       if ($this.data("state") === "on") {
           $this.removeClass("action-ficheform-Poids_on").addClass("action-ficheform-Poids_off");
           $this.attr("title", Bdf.Loc.get("_ link.edition.poids_off"));
           $this.data("state", "off");
           _display();
       } else {
           $this.removeClass("action-ficheform-Poids_off").addClass("action-ficheform-Poids_on");
           $this.attr("title", Bdf.Loc.get("_ link.edition.poids_on"));
           $this.data("state", "on");
           _save();
       }
    });
        
        function _display() {
            Bdf.$($itemList, {ficheformRole: "item-choice"}).each(function (index, element) {
                let $div = $(element);
                let $checkbox = Bdf.$($div, {ficheformRole: "item-checkbox"});
                let poids = Ficheform.getPoids($checkbox[0].value);
                $checkbox.after(Bdf.render("ficheform:poidsinput", {
                    poids: poids
                }));
            });
            Bdf.$($itemList, {ficheformRole: "poids"}).hide();
        }

        function _save() {
            Bdf.$($itemList, {ficheformRole: "item-choice"}).each(function (index, element) {
                let $div = $(element);
                let $poidsInput = Bdf.$($div, {ficheformRole: "poids-input"})
                let value = $poidsInput[0].value;
                let checkBox = Bdf.$($div, {ficheformRole: "item-checkbox"})[0];
                if (Ficheform.checkPoids(value)) {
                    Ficheform.updateCheckBox(checkBox, value);
                }
                $poidsInput.remove();
                Bdf.$($div, {ficheformRole: "poids"}).text(Ficheform.getPoidsString(checkBox.value)).show();
            });
        }
};

Ficheform.getCurrentCorpus = function (jqArgument) {
    if (!jqArgument) {
        return Bdf.$({ficheformRole: "form"}).data("ficheformCorpus");
    } else {
        return Bdf.$parents(jqArgument, {ficheformRole: "form"}).data("ficheformCorpus");
    }
};

Ficheform.initOrderButtons = function (rootId, element) {
    $$(rootId, "up").click(function () {
        let $previous = $(element).prev();
        if ($previous.length > 0) {
            $previous.before(element);
            Ficheform.signalChange(true);
        }
    });
    $$(rootId, "down").click(function () {
        let $next = $(element).next();
        if ($next.length > 0) {
            $next.after(element);
            Ficheform.signalChange(true);
        }
    });
};

Ficheform.initSearchInput = function (searchInputId, searchAnchorId, searchFunction) {
    $$(searchInputId).on("input", function () {
            searchFunction(this);
        }).on('keydown', function (keyEvent) {
            switch(keyEvent.key) {
                case "Enter": {
                    _getSelected().find("a").click();
                    keyEvent.stopPropagation();
                    keyEvent.preventDefault();
                    break;
                }
                case "ArrowDown": {
                    let $selected = _getSelected();
                    let $nextAll = $selected.nextAll("[data-ficheform-role='candidate']");
                    if ($nextAll.length > 0) {
                        $selected.removeClass("ficheform-search-Selected");
                        let $next = $($nextAll[0]);
                        $next.addClass("ficheform-search-Selected");
                        Bdf.ensureVisibility(Bdf.$parents($next, {ficheformRole:"search-popup"}), $next);
                    }
                    keyEvent.preventDefault();
                    break;
                }
                case "ArrowUp": {
                    let $selected = _getSelected();
                    let $previousAll = $selected.prevAll("[data-ficheform-role='candidate']");
                    if ($previousAll.length > 0) {
                        $selected.removeClass("ficheform-search-Selected");
                        let $previous = $($previousAll[0]);
                        $previous.addClass("ficheform-search-Selected");
                        Bdf.ensureVisibility(Bdf.$parents($previous, {ficheformRole:"search-popup"}), $previous);
                    }
                    keyEvent.preventDefault();
                    break;
                }
            }
        }).on("blur", function () {
            setTimeout(function () {$$(searchAnchorId).addClass("hidden");}, 250);
        }).on("focus", function () {
            $$(searchAnchorId).removeClass("hidden");
        }).attr("autocomplete", "off");
    
    function _getSelected() {
        return $$(searchAnchorId).find(".ficheform-search-Selected");
    }
};

Ficheform.delayFunction = function (id, delayedFunction) {
    clearTimeout(Ficheform.timerMap.get(id));
    let newTimer = setTimeout(delayedFunction, 300);
    Ficheform.timerMap.set(id, newTimer);
};
